﻿using BlackcmithWorkshopFileImplement;
using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopFileImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        private readonly DataFileSingleton _source;
        public ClientStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<ClientViewModel> GetFullList()
        {
            return _source.Clients
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ClientFIO) &&
                string.IsNullOrEmpty(model.Email) &&
                string.IsNullOrEmpty(model.Password))
            {
                return new();
            }
            return _source.Clients
                .Where(x =>
                            (string.IsNullOrEmpty(model.ClientFIO) || x.ClientFIO.Contains(model.ClientFIO)) &&
                            (string.IsNullOrEmpty(model.Email) || x.Email.Contains(model.Email)) &&
                            (string.IsNullOrEmpty(model.Password) || x.Password.Contains(model.Password)))
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            return _source.Clients
                .FirstOrDefault(x => 
                            (string.IsNullOrEmpty(model.ClientFIO) || x.ClientFIO == model.ClientFIO) &&
                            (string.IsNullOrEmpty(model.Email) || x.Email == model.Email) &&
                            (string.IsNullOrEmpty(model.Password) || x.Password == model.Password) &&
                            (!model.Id.HasValue || x.Id == model.Id))
                ?.GetViewModel;
        }
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = _source.Clients.Count > 0 ? _source.Clients.Max(x => x.Id) + 1 : 1;
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            _source.Clients.Add(newClient);
            _source.SaveClients();
            return newClient.GetViewModel;
        }
        public ClientViewModel? Update(ClientBindingModel model)
        {
            var client = _source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            _source.SaveClients();
            return client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            var element = _source.Clients.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                _source.Clients.Remove(element);
                _source.SaveClients();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
