﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopDatabaseImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        public List<ClientViewModel> GetFullList()
        {
            using var context = new BlacksmithWorkshopDataBase();
            return context.Clients
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ClientFIO) &&
                string.IsNullOrEmpty(model.Email) && 
                string.IsNullOrEmpty(model.Password))
            {
                return new();
            }
            using var context = new BlacksmithWorkshopDataBase();
            return context.Clients
                .Where(x =>
                            (string.IsNullOrEmpty(model.ClientFIO) || x.ClientFIO.Contains(model.ClientFIO)) &&
                            (string.IsNullOrEmpty(model.Email) || x.Email.Contains(model.Email)) &&
                            (string.IsNullOrEmpty(model.Password) || x.Password.Contains(model.Password)))
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ClientFIO) && 
                string.IsNullOrEmpty(model.Email) &&
                string.IsNullOrEmpty(model.Password) &&
                !model.Id.HasValue)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDataBase();
            return context.Clients
                .FirstOrDefault(x => (string.IsNullOrEmpty(model.ClientFIO) || x.ClientFIO == model.ClientFIO) && 
                                     (string.IsNullOrEmpty(model.Email) || x.Email == model.Email) &&
                                     (string.IsNullOrEmpty(model.Password) || x.Password == model.Password) &&
                                     (!model.Id.HasValue || x.Id == model.Id))
                ?.GetViewModel;
        }
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDataBase();
            context.Clients.Add(newClient);
            context.SaveChanges();
            return newClient.GetViewModel;
        }
        public ClientViewModel? Update(ClientBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var client = context.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            context.SaveChanges();
            return client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var element = context.Clients.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Clients.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
